/*
 * s t r u c t . h
 *
 * Define the structure of all antlr 2.0 data structures.
 */

#define StrSame			0
#define FIRST			1		/* Used as cache entry type */
#define FOLLOW			2

/* Only used in previous version of antlr (before 1.0B) */
#define NilStr	((char *) 1)	/* Not end of list, but not action; used in Actions */

#define D_NumAlts		10
#define D_NumAtoms		10
#define D_NumRules		10
#define Chunk			10

#define D_StrArraySize	20		/* Must be bigger than 0 */

/* A t o m  T y p e s */
#define aLeaf		1
#define aRule		2
#define aAction		3
#define aSubBLK		4
#define aOptBLK		5
#define aLoopBLK	6
#define EOL			7		/* End Of List */

#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif

#define MaxTokenSize	100

/* D a t a  S t r u c t u r e s */

struct atm {
	unsigned short type;
	unsigned short token;	/* Term-token or Rule # */
	union {
		char *action;		/* Points to an action */
		struct atm **block;	/* Points to a subblock */
		char *rule;			/* Points to rule name */
	} ptr;
	char *inheritance;		/* Do I inherit anything? (Rule or SubBlock) */
	struct atm *parent;		/* If blk, what blk am I nested in */
	int line;
	char *file;				/* Track for error information (usually for blks) */
};

typedef union {
			struct atm atom;
			struct atm *alt;
			struct atm **blk;
			char *inh;		/* Inheritance override for block or rule */
		} Attrib;

typedef struct atm Atom;
typedef Atom	*Alt;
typedef Alt		*Block;
typedef Block	*Rule;

/* M a c r o s */

#define markFi(r)	set_orel(r, &FiVisit);
#define markFo(r)	set_orel(r, &FoVisit);
#define unMarkFi(r)	set_rm(r, FiVisit);
#define unMarkFo(r)	set_rm(r, FoVisit);

/*
 * DEBUG macro
 * Bits 1..WORDSIZE-1 are set according to what the user
 * wants printed out.
 */
#define DEBUG(n)	if ( debug & (1<<(n-1)) ) 

/* E x p a n d a b l e  L i s t  M a c r o s */
#define sizeT(a)		*((unsigned int *) &((a)[0]))
#define makeBlkAtom(a,t,p,ln)										\
			(a).ptr.block=p; (a).type = t; (a).parent=NULL;			\
			(a).inheritance=NULL; (a).file = FileStr[CurFile];		\
			(a).line = ln;

#define InitStrArray(p)										\
	(p) = (char **) calloc(D_StrArraySize+1,sizeof(char *));\
	if ( (p) == NULL ) fatal("Can't Alloc Str Array");		\
	sizeT(p) = D_StrArraySize+1; p[1] = NULL;

#define InitRules(p)										\
	(p) = (Atom ***) calloc(D_NumRules+1,sizeof(Atom **));	\
	if ( (p) == NULL ) fatal("Can't Alloc Rules");			\
	sizeT(p) = D_NumRules+1;

#define InitRuleLine(p)		InitRuleFile(p)
#define InitRuleFile(p)										\
	(p) = (int *) calloc(D_NumRules+1,sizeof(int));			\
	if ( (p) == NULL ) fatal("Can't Alloc RuleFile");		\
	sizeT(p) = D_NumRules+1;

#define InitBlock(p)										\
	(p) = (Atom **) calloc(D_NumAlts+1, sizeof(Atom *));	\
	if ( (p) == NULL ) fatal("Can't Alloc Block");			\
	sizeT(p) = D_NumAlts+1;

#define InitAlt(p)										\
	(p) = (Atom *) calloc(D_NumAtoms+1, sizeof(Atom));	\
	if ( (p) == NULL ) fatal("Can't Alloc Alt");		\
	sizeT(p) = D_NumAtoms+1;

#define extAlt(a,i)										\
	if ( ++i == sizeT(a) )								\
	{													\
		(a) = (Atom *) realloc(a, (sizeT(a)+Chunk)*sizeof(Atom));	\
		if ( (a) == NULL ) fatal("Can't Extend Alt");	\
		sizeT(a) += Chunk;								\
	}

#define extBlk(a,i)										\
	if ( ++i == sizeT(a) )								\
	{													\
		(a) = (Atom **) realloc(a, (sizeT(a)+Chunk)*sizeof(Alt));	\
		if ( (a) == NULL ) fatal("Can't Extend Block");	\
		sizeT(a) += Chunk;								\
		if ( sizeT(a) > MaxAlts ) MaxAlts = sizeT(a);	\
	}

#define extRule(a,i)									\
	if ( ++i == sizeT(a) )								\
	{													\
		(a) = (Atom ***) realloc(a, (sizeT(a)+Chunk)*sizeof(Block));\
		if ( (a) == NULL ) fatal("Can't Extend Rules");	\
		sizeT(a) += Chunk;								\
	}

#define extRuleLine(a,i)		extRuleFile(a,i)
#define extRuleFile(a,i)								\
	if ( ++i == sizeT(a) )								\
	{													\
		(a) = (int *) realloc(a, (sizeT(a)+Chunk)*sizeof(int));\
		if ( (a) == NULL ) fatal("Can't Extend RuleFile");	\
		sizeT(a) += Chunk;								\
	}

#define extStrArray(a,i)								\
	if ( ++i == sizeT(a) )								\
	{													\
		register char **p;								\
		(a) = (char **) realloc(a, (sizeT(a)+Chunk)*sizeof(char *));\
		if ( (a) == NULL ) fatal("Can't Extend Rules");	\
		p = &(a[sizeT(a)]);								\
		sizeT(a) += Chunk;								\
		{	register int i;								\
			for (i=1; i<=Chunk; i++) *p++ = NULL; }		\
	}

#define defRule(i,r)	extRule(Rules,i); Rules[i] = r;
#define defRuleFile(i,f)	extRuleFile(RuleFile,i); RuleFile[i] = f;
#define defRuleLine(i,l)	extRuleLine(RuleLine,i); RuleLine[i] = l;
#define defBlk(a,i,b)	extBlk(a,i); (a)[i] = b;
#define defAlt(a,i,alt)	extAlt(a,i); (a)[i] = alt;

#define skipActions(p) while ( (*(p)).type == aAction ) {(p)++;}

/* E r r o r  r o u t i n e s */

#ifdef MPW		/* Macintosh Programmer's Workshop */
#define ErrHdr "File \"%s\"; Line %d #"
#else
#define ErrHdr "\"%s\", line %d:"
#endif

#define fatal(err)	fatalFL(err, __FILE__, __LINE__)
#define fatalFL(err, f, l)														\
			{fprintf(stderr, ErrHdr, f, l);										\
			fprintf(stderr, " %s\n", err);										\
			cleanUp();															\
			exit(-1);}
#define warn(err)	fprintf(stderr, "%s\n", err);
#define warnFL(err,f,l)															\
			{fprintf(stderr, ErrHdr, f, l);										\
			fprintf(stderr, " %s\n", err);}
#define warnFLNoCR( err,f,l )													\
			{fprintf(stderr, ErrHdr, f, l);										\
			fprintf(stderr, " warning: %s", err);}

#define warnBlk( a,err )														\
			{fprintf(stderr, ErrHdr, (a).file, (a).line);						\
			fprintf(stderr, " warning:");										\
			if ( BigErr ) dispAtom( (a), stderr );								\
			else fprintf(stderr, " subrule");									\
			fprintf(stderr, "%s\n", err);}
#define warnBlkNoCR( a,err )													\
			{fprintf(stderr, ErrHdr, (a).file, (a).line);						\
			fprintf(stderr, " warning:");										\
			if ( BigErr ) dispAtom( (a), stderr );								\
			else fprintf(stderr, " subrule");									\
			fprintf(stderr, "%s", err);}
#define errBlk( a,err )															\
			{fprintf(stderr, ErrHdr, (a).file, (a).line);						\
			if ( BigErr ) dispAtom( (a), stderr );								\
			else fprintf(stderr, " subrule");									\
			fprintf(stderr, "%s\n", err);}
#define setMsg( err, t )														\
			{fprintf(stderr, err);												\
			set_fpT(stderr, t);												\
			fprintf(stderr, "\n");}


/* E x t e r n s */

extern char		**TokenStr,
				**RegExprStr,
				**RuleStr,
				*FileStr[],
				**ActStr,
				**ActStr2,
				**FailActionStr,
				*eMsg(),
				*AttribAction,
				*LexFile;

extern set		FiVisit,
				FoVisit,
				RuleInh,
				*CrossRef;

extern int		MaxAlts;

extern Rule		Rules;
extern int		EpToken, EofToken, NumRules, NumFiles, CurFile, MaxToken;
extern int		ForceInherit, MakeTrax, Terminate, CR;

set 			*First(),
				Follow(),
				*fBlock(),
				fMidAlt(),
				followAlt(),
				followBlk(),
				fAlt(),
				fAtom(),
				*newFset(),
				*dupFset(),
				fCompact();
void			freeFset();

extern int		debug;
				
extern FILE		*NextFile();
extern int		*RuleFile, *RuleLine;
extern int		Save;
extern int		FoSave;
extern int		FoCompute;
extern int		FoPartial;
extern int		FoRequest;
extern int		UseCache;
extern int		BigErr, DfltErr;
extern int		NoCode,PrFirstSub,PrintFo,PrintFi;
extern set		*Cached();
extern unsigned char *set_bytes();
extern char		*TokenName(), *TokenValue();
extern char		*aSourceFile;

#ifndef MEMCHK
char *malloc(), *calloc(), *realloc();
#endif
