/* support routines for stringc
 * for ee595e reused for dlg
 *
 * Will Cohen
 * 9/17/89
 */

#include <stdio.h>
#include "dlg.h"

/* sets options from the command line */
options(argc, argv)
int argc;
char **argv;
{
	int errflg = FALSE;

	/* skip over command */
	++argv; -- argc;

	while (( argc ) && ( **argv == '-' )){
		switch ( (*argv)[1] ){
			case 'C': sscanf(&((*argv)[2]),"%d",&compress);
				  break;
			case 'D': sscanf(&((*argv)[2]),"%d",&num_of_dfa);
				  break;
			case 'N': sscanf(&((*argv)[2]),"%d",&num_of_nfa);
				  break;
			case '\0':
				  /* skip it, it could be stdin/stdout */
				  break;
			default:  errflg = TRUE;
				  break;
			}
		++ argv ; -- argc;
		}
	if (errflg)
		usage();
}

void fatal(message,line_no)
char *message;
int line_no;
{
	fprintf(stderr,"Fatal : %s, line : %d\n",message,line_no);
	exit(2);
}

void error(message,line_no)
char *message;
int line_no;
{
	fprintf(stderr,"\"%s\", line %d: %s\n",in_file_name,line_no,message);
}

void warning(message,line_no)
char *message;
int line_no;
{
	fprintf(stderr,"Warning : %s, line : %d\n",message,line_no);
}

void init_things()
{
	register int i;

	/* make one initalized nfa node */
	clear_nfa_node.accept = 0;
	clear_nfa_node.nfa_set = 0;
	clear_nfa_node.trans[0] = NIL_INDEX;
	clear_nfa_node.trans[1] = NIL_INDEX;
	clear_nfa_node.label = set_of(nil);
	

	/* make one initalized dfa node */
	clear_dfa_node.done = FALSE;
	clear_dfa_node.dfa_set = 0;
	for (i=0; i<=NUM_CLASS; ++i)
		clear_dfa_node.trans[i] = NIL_INDEX;

	/* make a set with the valid characters in it.
	   Used to mask off unused elements in set.
	   EOF is not included in this set since it is special.
	   All character shifted by one, so EOF can be element
	   zero in set. */
	{
		register	int i,
				first_el = FIRST_CHAR+1,
				last_el = LAST_CHAR+1;

		valid_chars = set_of(first_el);
		for (i=first_el; i<=last_el; ++i)
			valid_chars = set_or(valid_chars,set_of(i));
	}
}

usage()
{
	fprintf(stderr,"Usage: dlg [-cD#F#] infile outfile");
	fprintf(stderr,"\nDefault Meaning of option\n");
	fprintf(stderr,"C0\tno character class compression\n");
	fprintf(stderr,"C1\tonly compress out unused characters\n");
	fprintf(stderr,"C2\tdo character class compression\n");
	fprintf(stderr,"Dn\tn is maximum number of DFA nodes\n");
	fprintf(stderr,"Nn\tn is maximum number of NFA nodes\n");
	exit(1);
}

/* gets space for arrays and clears out symbol table entries */
init_arrays()
{
	register int	*p,*t;
	char *malloc();

	/* get arrays set up for dfa and nfa's */
	nfa_array = (nfa_node*) malloc(num_of_nfa * sizeof(nfa_node));
	if (!nfa_array)
		error("Unable to allocate space for nfa table.\n",0);
	dfa_array = (dfa_node*) malloc(num_of_dfa * sizeof(dfa_node));
	if (!dfa_array)
		error("Unable to allocate space for dfa table.\n",0);
	dfa_hash_table = (int*) malloc(hash_entries * sizeof(int));
	if (!dfa_hash_table)
		error("Unable to allocate space for dfa hash table.\n",0);
	t = &dfa_hash_table[hash_entries];
	for(p = dfa_hash_table; p < t; p++)
		*p = NIL_INDEX;
}
